declare module '@cip/request' {
  interface IResponseData<T>{
    code: number
    data: T
    message: string
  }
  interface IResponsePagingData<T>{
    code: number
    data: T
    message: string
    offset?: number
    pageNum?: number
    limit?: number
    total?: number
  }
  interface IAnyObject{
    [propname: string]: any
  }
  interface IPostOptions {
    form?: boolean
    autoNotify?: boolean
    headers?: IAnyObject
  }
  interface IGetOptions {
    autoNotify?: boolean
    headers?: IAnyObject
  }
  interface IPostParams {
    method: 'post'|'put'
    apiName: string
    url: string
    params?: IAnyObject
    pathParams?: IAnyObject
    data?: IAnyObject
    options?: IPostOptions
  }
  interface IGetParams{
    method?: 'get'|'delete' | undefined
    apiName: string
    url: string
    params?: IAnyObject
    pathParams?: IAnyObject
    options?:IPostOptions
  }


  // get
  function req(
    method: 'get'|'delete',
    apiName: string,
    url: string,
    params: IAnyObject,
    options?: IGetOptions | { pathParams?: IAnyObject}
  ): Promise<IResponsePagingData<any>>

  // post
  function req(
    method: 'post'|'put',
    apiName: string,
    url: string,
    data: IAnyObject,
    options?: IPostOptions | { pathParams?: IAnyObject, params?: IAnyObject,}
  ): Promise<IResponsePagingData<any>>
  // get
  function req(getParams: IGetParams | { method: 'get'|'delete'| undefined  }): Promise<IResponsePagingData<any>>
  // post
  function req(postParams: IPostParams | { method: 'post'|'put'  }): Promise<IResponsePagingData<any>>

  interface IFileRequestParams {
    apiName: string
    url: string
    data?: IAnyObject
    pathParams?: IAnyObject
    config?: IPostOptions
    headers?: IAnyObject
  }
  interface IUploadParams {
    method?: 'post'|'put'
    resolveCb?: (data: string|number, servicePath: string) => string
  }
  interface IDownloadParams{
    method?: 'post'|'put'|'get'|'delete'
  }
  export default req
  export const request = req
  export class UploadFile {
    constructor(uploadParams: IFileRequestParams & IUploadParams);
    send(): Promise<IResponseData<any>>
    abort(): void
  }
  export class DownloadFile {
    constructor(downloadParams: IFileRequestParams & IDownloadParams);
    send(): Promise<IResponseData<any>>
    abort(): void
  }
}
